﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.TravelingSalesman {
  /// <summary>
  /// An operator that performs similarity calculation between two traveling salesman solutions.
  /// </summary>
  [Item("TSPSimilarityCalculator", "An operator that performs similarity calculation between two traveling salesman solutions.")]
  [StorableClass]
  public sealed class TSPSimilarityCalculator : SimilarityCalculator {
    [StorableConstructor]
    private TSPSimilarityCalculator(bool deserializing) : base(deserializing) { }
    private TSPSimilarityCalculator(TSPSimilarityCalculator original, Cloner cloner) : base(original, cloner) { }
    public TSPSimilarityCalculator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new TSPSimilarityCalculator(this, cloner);
    }

    protected override double CalculateSimilarity(IScope left, IScope right) {
      Permutation v1 = left.Variables[TargetParameter.ActualName].Value as Permutation;
      Permutation v2 = right.Variables[TargetParameter.ActualName].Value as Permutation;

      int[,] edges = new int[v2.Length, 2];
      for (int i = 0; i < v2.Length; i++) {
        edges[v2[i], 0] = v2[(i + 1) % v2.Length];
        edges[v2[i], 1] = v2[(i - 1 + v2.Length) % v2.Length];
      }

      double similarity = 0.0;
      for (int i = 0; i < v1.Length; i++) {
        int targetCity = v1[(i + 1) % v1.Length];
        if (targetCity == edges[v1[i], 0] || targetCity == edges[v1[i], 1])
          similarity++;
      }

      return similarity / v1.Length;
    }
  }
}
